*//		Depending on where your dbfs, local views, 
*//		and remote views are, you will want to 
*//		make sure these are set correctly.
*//		Only DBC_NAME_DBF is used in these SPs.

#define DBC_NAME_DBF		sample
#define DBC_NAME_REMOTE		r_sample
#define DBC_NAME_LOCAL		l_sample

*//----------------------------------------------------
function sp_GetNewId( tcEntity )
	*//		Specific implementation for local FoxTables:
	local liNewId, lnSelect
	liNewId = .null.
	lnSelect = select()
	if not used('uniqueid')
		use uniqueid in 0
	endif
	select uniqueid
	locate for ENTITY = m.tcEntity
	if not found()
		locate for ENTITY = 'default'
		if not found()
			select (m.lnSelect)
			error 'uniqueid table does not contain a record for "'+m.tcEntity+'".'
			return m.liNewID
		endif
	endif	
	if rlock('uniqueid')
		liNewID = uniqueid.ID
		replace ID with ID + 1 ;
				in uniqueid
	
		unlock in uniqueid
	endif
	select (m.lnSelect)
	return m.liNewID
endfunc

*//------------------------------------------------------
function sp_SQLExecute( tcSQL, tcCursorName, tiFlags )

	local lcSQL, lcCursorName
	if type('m.tcCursorName') <> 'C'
		lcCursorName = 'sqlresult'
	else
		lcCursorName = m.tcCursorName
	endif

	lcSQL = m.tcSQL + ' into cursor ' + m.lcCursorName

	&lcSQL
	
	*//		Mimic the return values of SQLExec():
	if used( m.lcCursorname )
		return 1
	else
		return -1
	endif
endfunc

*//------------------------------------------------------
function sp_Pack( tcTableName )
	return sp_ExclusiveAction( m.tcTableName, 'P' )
endfunc

function sp_Zap( tcTableName )
	return sp_ExclusiveAction( m.tcTableName, 'Z' )
endfunc

*//------------------------------------------------------
function sp_ExclusiveAction( tcTableName, tcActionCode )
	local lcCurrentDBC, lnSelect, lcOnError, llSuccess, ;
	      llSafetyWasOn, llTableWasOpen
	llSuccess = .F.
	
	private plError		&& required PRIVATE because on ON.. trigger

	*//		Save the current environment, as near as possible:
	lcCurrentDBC 	= set('DATABASE')
	lnSelect 	 	= select()
	lcOnError		= on('ERROR')
	if set('SAFETY') = 'ON'
		llSafetyWasOn = .T.
		set safety off
	endif

	*//		Check for the table being open already:
	llTableWasOpen = used(m.tcTableName)
	if m.llTableWasOpen
		select (m.tcTableName)
	else
		set database to DBC_NAME_DBF
		select 0
	endif

	*//		Handle our own errors temporarily:
	on error store .T. to plError
	plError = .F.

	*//		Now open the table exclusively:
	use (m.tcTableName) excl
	if not m.plError
		*//		File must be open exclusively, so:		
		do case
			case m.tcActionCode = 'Z'
				zap
			case m.tcActionCode = 'P'
				pack
		endcase
		if not m.plError		
			llSuccess = .T.
		endif
	endif
	on error &lcOnError
	
	*//		Re-Open the table, if required:
	if m.llTableWasOpen
		use (m.tcTableName) shared			
	else
		use
	endif
	
	*//		Clean up:
	if m.llSafetyWasOn
		set safety on
	endif
	set database to (m.lcCurrentDBC)
	select (m.lnSelect)
	return m.llSuccess	
endfunc

*//------------------------------------------------------
function sp_Delete( tcTableName, tcForArgument )
	local lcCurrentDBC, lnSelect, lcOnError, llSuccess, llSafetyWasOn, llTableWasOpen
	llSuccess = .F.
	
	private plError		&& required PRIVATE because on ON.. trigger
	
	*//		Save the current environment, as near as possible:
	lcCurrentDBC 	= set('DATABASE')
	lnSelect 	 	= select()
	lcOnError		= on('ERROR')
	if set('SAFETY') = 'ON'
		llSafetyWasOn = .T.
		set safety off
	endif

	on error store .T. to plError   && 	Handle our own errors temporarily:
	plError = .F.

	*//		Check for the table being open already:
	llTableWasOpen = used(m.tcTableName)
	if m.llTableWasOpen
		select (m.tcTableName)
		if CURSORGETPROP("Buffering") > 1 then
			=TableUpdate( l_TABLEUPDATE_ALLROWS, l_TABLEUPDATE_FORCEOVERWRITE, m.tcTableName )			
		endif
	else
		set database to DBC_NAME_DBF
		select 0
		use (m.tcTableName)
	endif

	=CURSORSETPROP("Buffering", 1, m.tcTableName)   && Table should NOT be buffered anyway, so I won't reinstate orig mode
	if not m.plError
		if empty(m.tcForArgument) then
			delete ALL
		else
			delete for &tcForArgument
		endif
		if not m.plError		
			llSuccess = .T.
		endif
	endif
	on error &lcOnError
	
	if not m.llTableWasOpen      && Close table, if was not originally open
		use
	endif
	
	if m.llSafetyWasOn           && Clean up
		set safety on
	endif
	set database to (m.lcCurrentDBC)
	select (m.lnSelect)
	return m.llSuccess	
endfunc

*//------------------------------------------------------
*//	Take a field from a SPT-generated cursor and return
*//	the local (foxpro) date equivalent:
*// With FoxPro native DBC the SPT-generated cursor will have date type already so no conversion needed.
function sp_GetLocalDate
	lparameter tvDateTime
	return m.tvDateTime
endfunc

*//---------------------------------
function sp_GetDBType
	return 'FOXPRO'
endfunc

*//----------------------------------------------------
function sp_GetAvailableConnectionHandle()
	return 1
endfunc

*//----------------------------------------------------
function sp_GetNewConnectionHandle()
	return 1
endfunc

*//---------------------------------
function sp_IsConnectionBusy
	lparameter tcAlias
	*//		A local connection is never busy!
	return .F.
endfunc

*//----------------------------------------------------
function sp_WaitWhileConnectionBusy
	lparameter tcAlias
	return
endfunc

*//----------------------------------------------------
function sp_SQLCancel
	lparameter lvView
	return 1
endfunc

*//-------------------------------------------
function IsFlagSet
	lparameters tiFlags, tiSpecificFlag
	return (bitand( m.tiFlags, m.tiSpecificFlag ) > 0)
endfunc