*//		Depending on where your dbfs, local views, 
*//		and remote views are, you will want to 
*//		make sure these are set correctly.
*//		Only DBC_NAME_DBF is used in these SPs.

#define DBC_NAME_DBF		sample
#define DBC_NAME_REMOTE		r_sample
#define DBC_NAME_LOCAL		l_sample

*//		If you change these constants, 
*//		Remember to update the application
*//		 .H file as well!

#define CONNECTION_RETRY_TIMEOUT		0.5
#define SQLEXEC_FORCE_NEW_CONNECTION	2
#define SQLEXEC_NO_BUSY_CHECK           4

*//----------------------------------------------------
function sp_GetNewId( tcEntity )
	local liNewCode, lnSelect, lhConnection, liResult, lcSQL, llCloseConnection
	liNewCode 	      = .null.
	lnSelect  	      = select()
	liResult          = 0
	
	*//		Try for an existing connection:
	lhConnection = sp_GetAvailableConnectionHandle()
	if m.lhConnection > 0

		*//		We have a valid connection, but we 
		*//		don't close it after we're finished with it.

		llCloseConnection = .F.
	else
		*//		Create a new connection, and remember
		*//		to disconnect when we're done.

		lhConnection      = sp_GetNewConnectionHandle()
		llCloseConnection = (m.lhConnection > 0)
	endif

	if m.lhConnection > 0

		*//		Execute the sp_getnewid procedure on the server:

	    liResult = SQLExec( m.lhConnection, [exec sp_getnewid ?m.tcEntity] )

		if m.liResult > 0
			*//	If this worked, we should have a cursor 
			*//	called 'sqlresult':
			
			if used( 'sqlresult' )

				if not eof('sqlresult')
					liNewCode = sqlresult.ID
				else
					*//		No matching record - ie, entity key series
					*//		does not exist.

					error 'SYUID table does not contain a record for "'+m.tcEntity+'".'
					
				endif
				use in sqlresult
			else
				*//		The stored procedure did not execute correctly:
			endif
		else
			=aerrors(x)
			error x[1,2]			
		endif	
		
		if m.llCloseConnection
			=SQLDisconnect( m.lhConnection )
		endif
	endif

	select (m.lnSelect)	
	return m.liNewCode
	
endfunc

*//----------------------------------------------------
function sp_SQLExecute( tcSQL, tcCursorName, tiFlags )

	local lcSQL, lcCursorName, liFlags, lhConnection, liResult, llCloseConnection

	if type('m.tcCursorName') <> 'C'
		lcCursorName = 'sqlresult'
	else
		lcCursorName = m.tcCursorName
	endif
	
	if type('m.tiFlags')<>'N'
		liFlags = 0
	else
		liFlags = m.tiFlags
	endif

	lcSQL = strtran(m.tcSQL, '==','=')
	lcSQL = strtran(m.lcSQL, '.F.','0')
	lcSQL = strtran(m.lcSQL, '.T.','1')
	lcSQL = strtran(m.lcSQL, ["],['])

	if IsFlagSet( m.liFlags, SQLEXEC_FORCE_NEW_CONNECTION )
		lhConnection      = sp_GetNewConnectionHandle()
		llCloseConnection = (m.lhConnection > 0)
	else
		*//		Try for an existing connection:
	
		lhConnection = sp_GetAvailableConnectionHandle()
		if m.lhConnection > 0
			*//		We have a valid connection, but we 
			*//		don't close it after we're finished with it.

			llCloseConnection = .F.
		else
			*//		Create a new connection, and remember
			*//		to disconnect when we're done.

			lhConnection      = sp_GetNewConnectionHandle()
			llCloseConnection = (m.lhConnection > 0)
		endif
	endif

	if m.lhConnection > 0
	
		liResult = SQLExec( m.lhConnection, m.lcSQL, m.lcCursorName )

		if not IsFlagSet(m.liFlags, SQLEXEC_NO_BUSY_CHECK)
			
			do while sqlGetProp(m.lhConnection,'ConnectBusy')
				wait window "Connection is busy (sp_SQLExecute(" + m.lcCursorName + ")). Retrying..." ;
					timeout n_CONNECTION_RETRY_TIMEOUT
				if lastkey() = 27
					*// 	We escaped without waiting for the 
					*//		connection to become free.
					exit
				endif
			enddo
		endif
	
		if m.llCloseConnection
			=SQLDisconnect( m.lhConnection )
		endif
	
	endif
	return m.liResult 
endfunc

*//------------------------------------------------------
*//		Not applicable for SQL Server:
function sp_Pack
	lparameters tcTableName
	return .T.
endfunc

*//------------------------------------------------------
function sp_Zap
	lparameters tcTableName

	return sp_Delete( m.tcTablename )
endfunc

*//------------------------------------------------------
function sp_Delete
	lparameter tcTableName, tcFilter
	
	local lcSQL, lcFilter

	if empty( m.tcFilter )
		lcFilter = ""
	else
		lcFilter = "WHERE " + m.tcFilter
	endif

	lcSQL = "DELETE FROM " + m.tcTableName + " " + m.lcFilter

	if sp_SQLExecute( m.lcSQL ) > 0
		return .T.
	else
		return .F.
	endif

endfunc	

*//------------------------------------------------------
*//	Take a field from a SPT-generated cursor and return
*//	the local (foxpro) date equivalent:
function sp_GetLocalDate
	lparameter tvDateTime
	return ttod(m.tvDateTime)
endfunc

*//---------------------------------
function sp_GetDBType
	return 'SQLSERVER'
endfunc

*//----------------------------------------------------
function sp_GetAvailableConnectionHandle()

	local lhConnection
	lhConnection = -1
	
	*//	Look for a currently open remote view so
	*// that we can use its connection handle:
	
	for i = 1 to aused( laOpenView )
		if cursorgetprop('SourceType', laOpenView[i,1]) = 2		&& remote view
			lhConnection = cursorgetprop('ConnectHandle', laOpenView[i,1])
			exit
		endif
	endfor	
	return m.lhConnection
endfunc

*//----------------------------------------------------
function sp_GetNewConnectionHandle()

	*//		Open the connection directly. We can use
	*//		ADBOBJECTS() to find the connection without
	*//		knowing its name:

	if adbobjects( laConnect, 'CONNECTION' ) > 0
		
		*//		There will be only one connection defined
		*//		in this database, so use first array element:
		lhConnection = sqlConnect(laConnect[1])
	endif
	return m.lhConnection
endfunc

*//---------------------------------
function sp_IsConnectionBusy
	lparameter tcAlias

	local llCancelled, lhConnection, llCloseConnection
	llCancelled       = .F.
	llCloseConnection = .F.

	if empty( m.tcAlias)
		*//		Try for an existing connection:
		lhConnection = sp_GetAvailableConnectionHandle()

		if m.lhConnection < 1
			return .F.      &&  No connections therefore none busy !

*!*				*// Create a new connection, and remember
*!*				*// to disconnect when we're done.

*!*				*//---------------------------------------------
*!*				*//		Generally speaking, how valid is this?
*!*				*//		We're going to open a new connection
*!*				*//		so why the heck should it be busy?
*!*				*//		We even close it again afterwards!
*!*				*//---------------------------------------------

*!*				lhConnection      = sp_GetNewConnectionHandle()
*!*				llCloseConnection = (m.lhConnection > 0)
		endif
	else
		lhConnection = cursorgetprop('ConnectHandle',tcAlias)
	endif

	
	do while sqlGetProp(m.lhConnection,'ConnectBusy')

		*//	Clear the value of LastKey():
		keyboard chr(32) clear
		=inkey()

		wait window "Connection is busy (sp_IsConnectionBusy). Retrying..." ;
			timeout CONNECTION_RETRY_TIMEOUT
		if lastkey() = 27
			llCancelled = .T.
			exit
		endif
	enddo
*!*		if m.llCloseConnection
*!*			=SQLDisconnect( m.lhConnection )
*!*		endif
		
	return m.llCancelled
endfunc	

*//----------------------------------------------------
function sp_WaitWhileConnectionBusy
	lparameter tcAlias
	sp_IsConnectionBusy( m.tcAlias )
	return
endfunc

*//----------------------------------------------------
function sp_SQLCancel
	lparameter lvView
	local lhConnection
	if type('m.lvView') = 'N'
		lhConnection = m.lvView
	else
		lhConnection = cursorgetprop('ConnectHandle',m.lvView)
	endif
	return SQLCancel(m.lhConnection)
endfunc


*//-------------------------------------------
function IsFlagSet
	lparameters tiFlags, tiSpecificFlag
	return (bitand( m.tiFlags, m.tiSpecificFlag ) > 0)
endfunc